@@ -50,14 +50,7 @@
 
 #define DEBUG 0 /* 0 disable 1 enable  */
 
-// Local sensors type defines.
-#define VIA686A  0
-#define W83781D  1
-#define W83627HF 2
-#define AS99127F 3
-#define ADM1021  4
-
-#define OPT_STRING	"g:s:hH:w:m:M:a:e:u:t"
+#define OPT_STRING "g:s:hH:w:m:M:a:e:u:1:2:t:q"
 
 #define TEMPTOFAHRENHEIT(t) ((int)((t * (1.8) + 32)))
 #define TEMPTOKELVIN(t)     ((int)(t + 273))
@@ -88,7 +81,6 @@ inline void cycle_temptype();
 
 /* Globals */
 int delay      = 1;
-short chiptype = -1;	
 const sensors_chip_name *name;
 char *exec_app = NULL;
 
@@ -116,93 +108,73 @@ double execat       = 0;
 short  execed       = 0;
 short  swap_types   = 0;
 
+char *sensor_feature1 = "temp1";
+char *sensor_feature2 = "temp2";
+
+short quiet = 0;
+
 int main(int argc, char **argv) {
   char *chipname = NULL;
   int chip_nr = 0;
   int i = 0;
   short tmp_swap;
 
+  int feature1 = 0;
+  int feature2 = 0;
+  const sensors_feature_data* feature = NULL;
+  short chip_found = -1;
+
   BitOn(SENSOR_DISP, WARN_NONE);
   BitOn(SENSOR_DISP, TSCALE_CELCIUS);
   BitOn(SENSOR_DISP, GRAPH_LINE);
 
+  //  *conffname = "/etc/sensors.conf"; 
+
   if(!process_config(argc, argv)) {
     exit(-1);
   }
   
   if(!init_sensors()) {
-    printf("wmgtemp: Error initilising lm_sensors.");
     exit(-1);
   }
   
   /* Get the chip name */
   name = sensors_get_detected_chips(&chip_nr);
-  while(chiptype == -1 && name != NULL) {
-    if(!strcmp(name->prefix, SENSORS_VIA686A_PREFIX)) {
-      /* set the chip name */
-      chipname = SENSORS_VIA686A_PREFIX;
-      BitOn(SENSOR_DISP, CPU);
-      BitOn(SENSOR_DISP, SYS);
-      SENSOR_DEF_CPU = SENSORS_VIA686A_TEMP2;
-      SENSOR_DEF_SYS = SENSORS_VIA686A_TEMP;
-      chiptype = VIA686A;
-
-      /*
-      sensors_get_feature(*name, SENSORS_VIA686A_TEMP2_OVER, &cpu_limit);
-      sensors_get_feature(*name, SENSORS_VIA686A_TEMP_OVER, &sys_limit);
-      printf("cpu limit: %f", cpu_limit);
-      printf("sys limit: %f", sys_limit);
-      */
-    }
-    else if(!strcmp(name->prefix, SENSORS_ADM1021_PREFIX)) {
-      chipname = SENSORS_ADM1021_PREFIX;
-      BitOn(SENSOR_DISP, CPU);
-      BitOn(SENSOR_DISP, SYS);
-      SENSOR_DEF_CPU = SENSORS_ADM1021_REMOTE_TEMP;
-      SENSOR_DEF_SYS = SENSORS_ADM1021_TEMP;
-      chiptype = ADM1021;
-    }
-    else if(!strcmp(name->prefix, SENSORS_W83781D_PREFIX)) {
-      chipname = SENSORS_W83781D_PREFIX;
-      BitOn(SENSOR_DISP, CPU);
-      BitOn(SENSOR_DISP, SYS);
-      SENSOR_DEF_CPU = SENSORS_W83781D_TEMP2;
-      SENSOR_DEF_SYS = SENSORS_W83781D_TEMP1;
-      chiptype = W83781D;
-    }
-    else if(!strcmp(name->prefix, SENSORS_AS99127F_PREFIX)) {
-      chipname = SENSORS_AS99127F_PREFIX;
-      BitOn(SENSOR_DISP, CPU);
-      BitOn(SENSOR_DISP, SYS);
-      SENSOR_DEF_CPU = SENSORS_W83781D_TEMP2;
-      SENSOR_DEF_SYS = SENSORS_W83781D_TEMP1;
-      chiptype = AS99127F;
-    }
-    else if(!strcmp(name->prefix, SENSORS_W83627HF_PREFIX)) {
-      /* The lm_sensors W83781D driver is compatible with the Winbond W83627HF chip 
-	 so we'll use the same options, at least until the chips.h header file includes
-	 the W83627HF's defines -- we could provide these ourselves but they may 
-	 conflict with a later version of the sensors library */
-      chipname = SENSORS_W83627HF_PREFIX;
-      BitOn(SENSOR_DISP, CPU);
-      BitOn(SENSOR_DISP, SYS);
-      SENSOR_DEF_CPU = SENSORS_W83781D_TEMP2;
-      SENSOR_DEF_SYS = SENSORS_W83781D_TEMP1;
-      chiptype = W83627HF;
-    }
+  while(name != NULL && chip_found == -1) {
+    feature1 = feature2 = 0;
     
-    if(chiptype == -1) 
+    while((feature = sensors_get_all_features(*name, &feature1, &feature2)) != NULL) {
+      if(strcmp(feature->name, (const char *)sensor_feature1) == 0 && sensors_get_ignored(*name, feature->number)) {
+	SENSOR_DEF_CPU = feature->number;
+	BitOn(SENSOR_DISP, CPU);
+	chip_found = 1;
+      }
+      if(strcmp(feature->name, (const char *)sensor_feature2) == 0 && sensors_get_ignored(*name, feature->number)) {
+	SENSOR_DEF_SYS = feature->number;
+	BitOn(SENSOR_DISP, SYS);
+	chip_found = 1;
+      }
+    }
+
+    if(chip_found == 1) {
+      chipname = name->prefix;
+    }
+    else {
       name = sensors_get_detected_chips(&chip_nr);
+    }
   }
-  if(chiptype == -1) {
-    fprintf(stderr,"wmgtemp: Chip not supported.\n");
+  if(chip_found == -1) {
+    fprintf(stderr,"wmgtemp: Unable to find temperature sensing feature.\n");
     exit(0);
   }
 
   /* output the name of the sensor if found. */
-  printf("wmgtemp: Primary Sensor - %s on %s\n", name->prefix, sensors_get_adapter_name(name->bus));
+  if(quiet == 0)
+    printf("wmgtemp: Primary Sensor - %s on %s\n", name->prefix, sensors_get_adapter_name(name->bus));
+  
   if(swap_types) {
-    printf("wmgtemp: swapping temps\n");
+    if(quiet == 0)
+      printf("wmgtemp: swapping temps\n");
     tmp_swap = SENSOR_DEF_SYS;
     SENSOR_DEF_SYS = SENSOR_DEF_CPU;
     SENSOR_DEF_CPU = tmp_swap;
@@ -485,31 +457,39 @@ int recompute_range(double cpu_high, double cpu_low, double sys_high, double sys
   
   // --------
   if(IsOn(SENSOR_DISP, CPU) && IsOn(SENSOR_DISP, SYS)) {
-    if((cpu_high < (range_upper - range_step) && sys_high < (range_upper - range_step)) && (range_upper - range_step) >= display_max) {
+    if((cpu_high < (range_upper - range_step) && 
+	sys_high < (range_upper - range_step)) && 
+       (range_upper - range_step) >= display_max) {
       range_upper -= range_step;
       modified = 1;
     }
-    if((cpu_low > (range_lower + range_step) && sys_low > (range_lower + range_step)) && (range_lower + range_step) <= display_min ) {  
+    if((cpu_low > (range_lower + range_step) && 
+	sys_low > (range_lower + range_step)) && 
+       (range_lower + range_step) <= display_min ) {  
       range_lower += range_step;  
       modified = 1;  
     }  
   }
   else if(IsOn(SENSOR_DISP, CPU) && !IsOn(SENSOR_DISP, SYS)) {
-    if(cpu_high < (range_upper - range_step) && (range_upper - range_step) >= display_max) {
+    if(cpu_high < (range_upper - range_step) && 
+       (range_upper - range_step) >= display_max) {
       range_upper -= range_step;
       modified = 1;
     }    
-    if(cpu_low > (range_lower + range_step) && (range_lower + range_step) <= display_min) { 
+    if(cpu_low > (range_lower + range_step) && 
+       (range_lower + range_step) <= display_min) { 
       range_lower += range_step;  
       modified = 1;  
     } 
   }
   else if(!IsOn(SENSOR_DISP, CPU) && IsOn(SENSOR_DISP, SYS)) {
-    if(sys_high < (range_upper - range_step) && (range_upper - range_step) >= display_max) {
+    if(sys_high < (range_upper - range_step) && 
+       (range_upper - range_step) >= display_max) {
       range_upper -= range_step;
       modified = 1;
     }    
-    if(sys_low > (range_lower + range_step) && (range_lower + range_step) <= display_min) { 
+    if(sys_low > (range_lower + range_step) && 
+       (range_lower + range_step) <= display_min) { 
       range_lower += range_step;  
       modified = 1;  
     }
@@ -595,53 +575,55 @@ int init_sensors() {
   FILE *config_file;
   int res;
 
-  /* Initialise sensors using sensors config file */
   config_file = fopen("/etc/sensors.conf", "r");
+
+  if(config_file == NULL) {
+    fprintf(stderr, "Error opening /etc/sensors.conf\n");
+    return 0;
+  }
+
   res = sensors_init(config_file);
 
   if(res != 0) {
-    config_file = fopen("/usr/etc/sensors.conf", "r");
-    res = sensors_init(config_file);
-
-    if(res != 0) {
-      config_file = fopen("/usr/local/etc/sensors.conf", "r");
-      res = sensors_init(config_file);
-
-      if(res != 0) {
-	fprintf(stderr,"%s\n", sensors_strerror(res));
-	return 0;
-      }
-    }
+    fprintf(stderr,"Error initializing sensors: %s\n", sensors_strerror(res));
+    return 0;
   }
-  fclose(config_file);
+
+  if(fclose(config_file))
+    perror("Error closing sensors.conf");
+
   return 1;
 }
 
 void display_usage() {
-  printf("Usage: wmgtemp [options]
-Options:
-   -s, --scale=SCALE  display temperatures in SCALE
-                      SCALE=kelvin, fahrenheit
-                      [Default: celcius]
-   -g, --graph=STYLE  display graph as STYLE
-                      STYLE=line, block
-                      [Default: line]
-   -H, --high=TEMP    Display red warning light at TEMP degrees celcius
-                      [Default: 50]
-   -w, --warn=TEMP    Display amber warning light at TEMP degrees celcius
-                      [Default: 45]
-   -u, --update=SEC   update the display every SEC seconds
-                      [Default: 1]
-   -m, --min=TEMP     set the lower bound of the graph to TEMP degrees celcius
-                      [Default: 20]
-   -M, --max=TEMP     set the upper bound of the graph to TEMP degrees celcius
-                      [Default: 35]
-   -a, --execat=TEMP  Execute a command at TEMP degrees celcius
-   -e, --exec=COMMAND Execute COMMAND when the 'execat' temperature is reached
-   -t, --swap         Swap CPU and SYS temps
-   -h, --help         Displays this help screen
-");
-  
+  printf("wmgtemp v0.7\n" \
+	 "Usage: wmgtemp [options]\n" \
+	 "Options:\n" \
+	 "   -s, --scale=SCALE  display temperatures in SCALE\n" \
+	 "                      SCALE=kelvin, fahrenheit\n" \
+	 "                      [Default: celcius]\n" \
+	 "   -g, --graph=STYLE  display graph as STYLE\n" \
+	 "                      STYLE=line, block\n" \
+	 "                      [Default: line]\n" \
+	 "   -H, --high=TEMP    Display red warning light at TEMP degrees celcius\n" \
+	 "                      [Default: 50]\n" \
+	 "   -w, --warn=TEMP    Display amber warning light at TEMP degrees celcius\n" \
+	 "                      [Default: 45]\n" \
+	 "   -u, --update=SEC   update the display every SEC seconds\n" \
+	 "                      [Default: 1]\n" \
+	 "   -m, --min=TEMP     set the lower bound of the graph to TEMP degrees celcius\n" \
+	 "                      [Default: 20]\n" \
+	 "   -M, --max=TEMP     set the upper bound of the graph to TEMP degrees celcius\n" \
+	 "                      [Default: 35]\n" \
+	 "   -1, --feature1=F1  set the feature for CPU\n" \
+	 "                      [Default: temp1]\n" \
+	 "   -2, --feature2=F2  set the feature for SYS\n" \
+	 "                      [Default: temp2]\n" \
+	 "   -a, --execat=TEMP  Execute a command at TEMP degrees celcius\n" \
+	 "   -e, --exec=COMMAND Execute COMMAND when the 'execat' temperature is reached\n" \
+	 "   -t, --swap         Swap CPU and SYS temps\n" \
+	 "   -q, --quiet        Don't display any messages\n" \
+	 "   -h, --help         Displays this help screen\n");
 }
 
 void draw_warning_lights(double current_temp) {
@@ -734,6 +716,9 @@ int process_config(int argc, char **argv) {
   char *rc_exec   = NULL;
   char *rc_delay  = NULL;
   char *rc_swap   = NULL;
+  char *rc_feature1 = NULL;
+  char *rc_feature2 = NULL;
+  char *rc_quiet = NULL;
   short parse_ok  = 1;
   int opt_index;
   int opt;
@@ -751,6 +736,9 @@ int process_config(int argc, char **argv) {
     { "exec", &rc_exec },
     { "update", &rc_delay },
     { "swap", &rc_swap },
+    { "quiet", &rc_quiet },
+    { "feature1", &rc_feature1 },
+    { "feature2", &rc_feature2 },
     { NULL, NULL }
   };
   
@@ -764,7 +752,10 @@ int process_config(int argc, char **argv) {
     {"execat", required_argument, 0, 'a'},
     {"exec",   required_argument, 0, 'e'},
     {"update", required_argument, 0, 'u'},
+    {"feature1", required_argument, 0, '1'},
+    {"feature2", required_argument, 0, '2'},
     {"swap",   no_argument,       0, 't'},
+    {"quiet",   no_argument,       0, 'q'},
     {"help",   no_argument,       0, 'h'},
     {0, 0, 0, 0}
   };
@@ -804,6 +795,15 @@ int process_config(int argc, char **argv) {
     case 'u':
       rc_delay = strdup(optarg);
       break;
+    case '1':
+      rc_feature1 = strdup(optarg);
+      break;
+    case '2':
+      rc_feature2 = strdup(optarg);
+      break;
+    case 'q':
+      rc_quiet = "y";
+      break;
     case 't':
       rc_swap = "y";
       break;
@@ -816,6 +816,19 @@ int process_config(int argc, char **argv) {
     }
   }
 
+  if(rc_quiet != NULL) {
+    if(!strncmp(rc_quiet, "y", 1)) {
+      quiet = 1;
+    }
+  }
+
+  if(rc_feature1 != NULL) {
+    sensor_feature1 = strdup(rc_feature1);
+  }
+  if(rc_feature2 != NULL) {
+    sensor_feature2 = strdup(rc_feature2);
+  }
+
   if(rc_graph != NULL) {
     if(!strncmp(rc_graph, "l", 1)) {
       BitOff(SENSOR_DISP, GRAPH_BLOCK);
@@ -856,7 +869,8 @@ int process_config(int argc, char **argv) {
       parse_ok = 0;
     }
     else {
-      printf("wmgtemp: high temp set to %d degrees celcius.\n", (int)high_temp);
+      if(quiet == 0)
+	printf("wmgtemp: high temp set to %d degrees celcius.\n", (int)high_temp);
     }
   }
   if(rc_warn != NULL) {
@@ -866,7 +880,8 @@ int process_config(int argc, char **argv) {
       parse_ok = 0;
     }
     else {
-      printf("wmgtemp: warning temp set to %d degrees celcius.\n", (int)warn_temp);
+      if(quiet == 0)
+	printf("wmgtemp: warning temp set to %d degrees celcius.\n", (int)warn_temp);
     }
   }
   if(rc_max != NULL) {
@@ -876,7 +891,8 @@ int process_config(int argc, char **argv) {
       parse_ok = 0;
     }
     else {
-      printf("wmgtemp: Upper range set to %d degrees celcius.\n", (int)range_upper);
+      if(quiet == 0)
+	printf("wmgtemp: Upper range set to %d degrees celcius.\n", (int)range_upper);
     }
   }
   if(rc_min != NULL) {
@@ -886,7 +902,8 @@ int process_config(int argc, char **argv) {
       parse_ok = 0;
     }
     else {
-      printf("wmgtemp: Lower range set to %d degrees celcius.\n", (int)range_lower);
+      if(quiet == 0)
+	printf("wmgtemp: Lower range set to %d degrees celcius.\n", (int)range_lower);
     }
   }
   if(rc_delay != NULL) {
@@ -896,7 +913,8 @@ int process_config(int argc, char **argv) {
       parse_ok = 0;
     }
     else {
-      printf("wmgtemp: update delay set to %d seconds.\n", delay);
+      if(quiet == 0)
+	printf("wmgtemp: update delay set to %d seconds.\n", delay);
     }
   }
   if(rc_execat != NULL) {
@@ -937,3 +955,4 @@ int process_config(int argc, char **argv) {
 
   return parse_ok;
 }
+
